import { CachedContent, GraphQLConfig } from 'graphql-language-service';
import { GraphQLLanguageService } from './GraphQLLanguageService';
import type { CompletionParams, DidSaveTextDocumentParams, DidOpenTextDocumentParams, DidChangeConfigurationParams, Diagnostic, CompletionList, CancellationToken, Hover, InitializeResult, Location, PublishDiagnosticsParams, DidChangeTextDocumentParams, DidCloseTextDocumentParams, DidChangeWatchedFilesParams, InitializeParams, Range as RangeType, TextDocumentPositionParams, DocumentSymbolParams, SymbolInformation, WorkspaceSymbolParams, Connection, DidChangeConfigurationRegistrationOptions } from 'vscode-languageserver/node';
import { GraphQLCache } from './GraphQLCache';
import { parseDocument } from './parseDocument';
import { GraphQLExtensionDeclaration } from 'graphql-config';
import type { LoadConfigOptions } from './types';
import { SupportedExtensionsEnum } from './constants';
import { NoopLogger, Logger } from './Logger';
declare type CachedDocumentType = {
    version: number;
    contents: CachedContent[];
};
export declare class MessageProcessor {
    _connection: Connection;
    _graphQLCache: GraphQLCache;
    _graphQLConfig: GraphQLConfig | undefined;
    _languageService: GraphQLLanguageService;
    _textDocumentCache: Map<string, CachedDocumentType>;
    _isInitialized: boolean;
    _isGraphQLConfigMissing: boolean | null;
    _willShutdown: boolean;
    _logger: Logger | NoopLogger;
    _extensions?: GraphQLExtensionDeclaration[];
    _parser: (text: string, uri: string) => CachedContent[];
    _tmpDir: string;
    _tmpUriBase: string;
    _tmpDirBase: string;
    _loadConfigOptions: LoadConfigOptions;
    _schemaCacheInit: boolean;
    _rootPath: string;
    _settings: any;
    constructor({ logger, fileExtensions, graphqlFileExtensions, loadConfigOptions, config, parser, tmpDir, connection, }: {
        logger: Logger | NoopLogger;
        fileExtensions: ReadonlyArray<SupportedExtensionsEnum>;
        graphqlFileExtensions: string[];
        loadConfigOptions: LoadConfigOptions;
        config?: GraphQLConfig;
        parser?: typeof parseDocument;
        tmpDir?: string;
        connection: Connection;
    });
    get connection(): Connection;
    set connection(connection: Connection);
    handleInitializeRequest(params: InitializeParams, _token?: CancellationToken, configDir?: string): Promise<InitializeResult>;
    _updateGraphQLConfig(): Promise<void>;
    private _handleConfigError;
    private _logConfigError;
    private _isGraphQLConfigFile;
    handleDidOpenOrSaveNotification(params: DidSaveTextDocumentParams | DidOpenTextDocumentParams): Promise<PublishDiagnosticsParams>;
    handleDidChangeNotification(params: DidChangeTextDocumentParams): Promise<PublishDiagnosticsParams | null>;
    handleDidChangeConfiguration(_params: DidChangeConfigurationParams): Promise<DidChangeConfigurationRegistrationOptions>;
    handleDidCloseNotification(params: DidCloseTextDocumentParams): void;
    handleShutdownRequest(): void;
    handleExitNotification(): void;
    private validateDocumentAndPosition;
    handleCompletionRequest(params: CompletionParams): Promise<CompletionList>;
    handleHoverRequest(params: TextDocumentPositionParams): Promise<Hover>;
    handleWatchedFilesChangedNotification(params: DidChangeWatchedFilesParams): Promise<Array<PublishDiagnosticsParams | undefined> | null>;
    handleDefinitionRequest(params: TextDocumentPositionParams, _token?: CancellationToken): Promise<Array<Location>>;
    handleDocumentSymbolRequest(params: DocumentSymbolParams): Promise<Array<SymbolInformation>>;
    handleWorkspaceSymbolRequest(params: WorkspaceSymbolParams): Promise<Array<SymbolInformation>>;
    private _getTextDocuments;
    private _cacheSchemaText;
    private _cacheSchemaFile;
    private _getTmpProjectPath;
    private _cacheSchemaFilesForProject;
    private _cacheConfigSchema;
    private _cacheDocumentFilesforProject;
    private _cacheAllProjectFiles;
    _isRelayCompatMode(query: string): boolean;
    private _updateFragmentDefinition;
    private _updateSchemaIfChanged;
    private _unwrapProjectSchema;
    private _globIfFilePattern;
    private _updateObjectTypeDefinition;
    private _getCachedDocument;
    private _invalidateCache;
}
export declare function processDiagnosticsMessage(results: Diagnostic[], query: string, range: RangeType | null): Diagnostic[];
export {};
//# sourceMappingURL=MessageProcessor.d.ts.map